#ifndef	AFS_PARAM_H
#define	AFS_PARAM_H

#define __alpha 1
#define	AFS_ALPHA_ENV	1
#define AFS_ALPHA_XBSD_ENV 1

#define AFS_64BIT_ENV	1

#define SYS_NAME       "alpha_nbsd15"
#define SYS_NAME_ID    SYS_NAME_ID_alpha_nbsd15

#define AFSLITTLE_ENDIAN 1

#endif	/* AFS_PARAM_COMMON_H */
