#ifndef	AFS_PARAM_COMMON_H
#define	AFS_PARAM_COMMON_H

#include <sys/param.h>

#define AFS_X86_XBSD_ENV 1

#define AFS_X86_ENV 1

#define SYS_NAME       "i386_nbsd15"
#define SYS_NAME_ID    SYS_NAME_ID_i386_nbsd15

#define AFSLITTLE_ENDIAN 1

#endif	/* AFS_PARAM_COMMON_H */
